#include "mbed.h"
#include "pinmap.h"

// square pin will be used to generate the pulses to the TPM external clock source
DigitalOut square(PTB8);

// using the Blue Led for visual interface
DigitalOut led1(LED_BLUE);

// serial ... just to try it
Serial pc(USBTX, USBRX);

// TPM_Type structure and all the macros used are declared in MKL25Z4.h 
// using a global pointer
TPM_Type *tpmobj;

void setup_tpm(void)
{
 // TPM_Type structure is declared in MKL25Z4.h 
 // several locations, one of those: http://developer.mbed.org/users/mbed_official/code/mbed/file/487b796308b0/TARGET_KL25Z/MKL25Z4.h
 // give it the address of TPM Module 0
 tpmobj=(TPM_Type *)TPM0_BASE;


       
    // making sure all clock gating are active
    SIM->SCGC5 |= (1 << SIM_SCGC5_PORTE_SHIFT);  //PORTE clock gating ---- no need!
    SIM->SCGC6 |= (1 << SIM_SCGC6_TPM0_SHIFT);   // TPM0 clock gating
    
    // Select TPM clock source in System Options Register
    SIM->SOPT2 |= SIM_SOPT2_TPMSRC(2); //OSCERCLK clock
    
    // Select TPM0 External Pin to drive the clock to TPM0 module as TPM_CLKIN0 (bit 24=0)
    // TPM external clock funtion will be set to 4 by using pin_function()
    SIM->SOPT4 &= ~(1<<SIM_SOPT4_TPM0CLKSEL_SHIFT); 
    
    
    
    // clear the overflow flag (bit 7 to 1), use MKL25Z4.h macros to write bits CMOD[] and PS[] with desired values
    tpmobj->SC=(1<<7) | TPM_SC_CMOD(1) | TPM_SC_PS(6);
    
    //first take care of the counter
    tpmobj->CNT=0;
    //them initialize the modulo
    tpmobj->MOD=30000u;    
    //*(aux+8)=30000u;
    //*(aux+8)=10u; // counter
    
    // Last wire pinout - make sure PTE29 is function 4 in order to act has External Clock input (TPM_CLKIN0)
    pin_function(PTE29, 4);

    //pin_mode(PTE29, PullUp); //GPIO input
    
}

int main(void) {
   
  
    led1=1;
    pc.baud(9600);
    

    setup_tpm();
    led1=0;
    printf("testing ...\n");
  
    while(1==1)
     {
          square=!square;
          //polling for TPM0 Overflow 
          if(tpmobj->SC & TPM_SC_TOF_MASK)  
          {
              led1=!led1;
              //Clear the overflow flag by writing 1 to bit 7 of SC
              tpmobj->SC|=TPM_SC_TOF_MASK;
          }
          //wait(0.1);    
     }
}